#!/bin/bash
#
# Name: NTPTest.sh
# Date: Feb 2018  
# Author: Zvan Miha <zvan@iskratel.si>
#
# This is test script for supervising NTP service
# Test starts from diagnostic module, and it is periodicaly executed
# Period depends on settings in test_program.dat.
#
# First, test will check if this is systemd or systemV type of Linux
# On systemV we have oldfashion ntpd, but on systemd we have chronyd
#
# Next, test will check if NTP service is running. This is done via pidof.
# if not test will report alarm 1300040    /*NTP synchronization not running */
# On systemd, init will restart chronyd service. But on systemV test restarts ntpd
# via /opt/iskratel/etc/init.d/it_ntpd.sh start
# Alarm will be retreated, when service is running
#
# Next, test will check if server is listed in configuration
# On systemV will check /etc/ntp.conf. If this file dont exists or, if there
# are no servers listed, test will report alarm 1300060    /*System configuration error */
# Same will be done for chronyd in systemd, except test will check /etc/chrony.conf.
# Alarm will be retreated, when configuration file will exists, with servers listed.
#
# Next, test will check if NTP server is synchronized.
# On systemV, this will be done via ntpq -p. 
# On systemd, this will be done via chronyc status. There is also mandatory second
# check, which invokes timedatectl command.
# If NTP is not synchronized, test will report alarm 1300030 /*NTP server is not responding */
# Alarm is retreated, when NTP server is synchronized
#
# If configuration in configuration file (/etc/ntp.conf or /etc/chrony.conf) has changed,
# then test will restart NTP server
#
#
# Some important files are on /tmp directory
# if we want to debug this test, just create file /tmp/NTPdebug
# 
# Return values
#	0 - OK
#
##############################################################
#ntpq -p output
#Columns Defined:
#remote: peers speficified in the ntp.conf file
#* = current time source
## = source selected, distance exceeds maximum value
#o = source selected, Pulse Per Second (PPS) used
#+ = source selected, included in final set
#x = source false ticker
#. = source selected from end of candidate list
#_ = source discarded by cluster algorithm
#blank = source discarded high stratum, failed sanity

#chronyc sources explained
#The state of the sources.
#* indicates the source to which chronyd is currently synchronized.
#+ indicates acceptable sources that are combined with the selected source.
#- indicates acceptable sources that are excluded by the combining algorithm.
#? indicates sources to which connectivity has been lost or whose packets do not pass all tests.
#x indicates a clock that chronyd thinks is a false ticker, that is, its time is inconsistent with a majority of other sources.
#~ indicates a source whose time appears to have too much variability.
#The .?. condition is also shown at start-up, until at least three samples have been gathered from it.


systemd=undefined
systemV=undefined
systemType=undefined
LOG_FILE=/tmp/NTPdebug.log
export LOGGER="logger -s -p user.alert -t NTPTest.sh"


#####################
# F U N C T I O N S #
#####################
# Check if chronyd is running and handle alarm
function chronyd_is_running()
{
	#Get chronyd PID
	ChronyPID=$(pidof chronyd)

	#On systemd we are running chronyd
	if [ "$ChronyPID" == "" ]
	then
		# Report Alarm 1300040 - NTP synchronization not running, if not allready reported
		if [ ! -f /tmp/1300040_alarm_on ]
		then
			if [ -e /opt/iskratel/bin/diagtool ]
			then
				# Report alarm and mark it
				/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300040 -idNum 0 0 -add chronyd 2>/dev/null >/dev/null
			else
				echo "No diagtool installed on system - problem reporting / retreating alarm" | $LOGGER 2>/dev/null
			fi

			touch /tmp/1300040_alarm_on
			if [ -f /tmp/1300040_alarm_off ]
			then
				rm -f /tmp/1300040_alarm_off
			fi

			if [ "$DEBUG" == "yes" ]
			then
				echo "Server chronyd is not running - report alarm 1300040" >>$LOG_FILE
			fi

			#On systemd we manualy restart chronyd
			if type systemctl >/dev/null 2>/dev/null
			then
				systemctl start chronyd
			else
				/etc/init.d/chronyd start
				sleep 5
			fi

		fi
	else
		#Server is running - if alarm was reported, clear it
		if [ ! -f /tmp/1300040_alarm_off ]
		then
			if [ -e /opt/iskratel/bin/diagtool ]
	                then
				# Retreat alarm and mark it
				/opt/iskratel/bin/diagtool -env -ret -norpc -ac 1300040 -idNum 0 0 -add chronyd 2>/dev/null >/dev/null
			else
			        echo "No diagtool installed on system - problem reporting / retreating alarm" | $LOGGER 2>/dev/null
			fi

			touch /tmp/1300040_alarm_off
			if [ -f /tmp/1300040_alarm_on ]
			then
				rm -f /tmp/1300040_alarm_on
			fi

			if [ "$DEBUG" == "yes" ]
                	then
				echo "Server chronyd is running - retreat alarm 1300040" >>$LOG_FILE
			fi
		fi
	fi
	return 0
}



# Check if ntpd is running and handle alarm
# If ntpd is not running - start it
function ntpd_is_running()
{
        #Get chronyd PID
	NtpdPID=$(pidof ntpd)

	#On systemV we are running ntpd
	if [ "$NtpdPID" == "" ]
	then
		# Report Alarm 1300040 - NTP synchronization not running, if not allready reported
		if [ ! -f /tmp/1300040_alarm_on ]
		then
			if [ -e /opt/iskratel/bin/diagtool ]
			then
				# Report alarm and mark it
				/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300040 -idNum 0 0 -add ntpd 2>/dev/null >/dev/null
			else
			        echo "No diagtool installed on system - problem reporting / retreating alarm" | $LOGGER 2>/dev/null
			fi

			touch /tmp/1300040_alarm_on
			if [ -f /tmp/1300040_alarm_off ]
			then
				rm -f /tmp/1300040_alarm_off
			fi

			if [ "$DEBUG" == "yes" ]
			then
				echo "Server ntpd is not running - report alarm 1300040" >>$LOG_FILE
			fi

			#On systemV we manualy restart ntpd
			restart_ntpd
		fi
	else
		#Server is running - if alarm was reported, clear it
		if [ ! -f /tmp/1300040_alarm_off ]
		then
			if [ -e /opt/iskratel/bin/diagtool ]
	                then
				# Retreat alarm and mark it
				/opt/iskratel/bin/diagtool -env -ret -norpc -ac 1300040 -idNum 0 0 -add ntpd 2>/dev/null >/dev/null
			else
			        echo "No diagtool installed on system - problem reporting / retreating alarm" | $LOGGER 2>/dev/null
			fi

			touch /tmp/1300040_alarm_off
			if [ -f /tmp/1300040_alarm_on ]
			then
				rm -f /tmp/1300040_alarm_on
			fi
	
			if [ "$DEBUG" == "yes" ]
       	         	then
				echo "Server ntpd is running - retreat alarm 1300040" >>$LOG_FILE
			fi
		fi
	fi
	return 0
}



#Check if we have any servers listed. Do this via /etc/chrony.conf
function chronyd_servers_listed()
{
	if [ -f /etc/chrony.conf ]
	then
		# Configuration file exists
		server_num_all=$(cat /etc/chrony.conf | grep server | wc -l)
		server_num_hashed=$(cat /etc/chrony.conf | grep server | awk '{print $1;}' | grep "#" | wc -l)
		server_num=$(echo $(($server_num_all - $server_num_hashed)))

		if [ $server_num -eq 0 ]
		then
			# there are no servers in configuration file
			if [ ! -f /tmp/1300060_alarm_on ]
			then
				if [ -e /opt/iskratel/bin/diagtool ]
				then
					#Report alarm for missing configuration file"
					/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300060 -idNum 0 0 -add "Wrong /etc/chrony.conf" 2>/dev/null >/dev/null
				else
			        	echo "No diagtool installed on system - problem reporting / retreating alarm" | $LOGGER 2>/dev/null
				fi

				touch /tmp/1300060_alarm_on
				if [ -f /tmp/1300060_alarm_off ]
				then
					rm -f /tmp/1300060_alarm_off
				fi

				if [ "$DEBUG" == "yes" ]
				then
					echo "No servers in configuration file - /etc/chrony.conf" >>$LOG_FILE
				fi

				#Clear previously reported 1300030 if exists
				if [ -f /tmp/1300030_alarm_on ]
	                        then
					/opt/iskratel/bin/diagtool -env -ret -norpc -ac 1300030 -idNum 0 0 -add "Cleared because 1300060 was reported" 2>/dev/null >/dev/null	
					mv /tmp/1300030_alarm_on /tmp/1300030_alarm_off

					if [ "$DEBUG" == "yes" ]
	                                then
						echo "Retreate previously reported 1300030" >>$LOG_FILE
					fi
				fi
			fi
		else
			# Configuration file is OK
			if [ ! -f /tmp/1300060_alarm_off ]
			then
				if [ -e /opt/iskratel/bin/diagtool ]
				then
					#Retreat alarm for missing configuration file"
					/opt/iskratel/bin/diagtool -env -ret -norpc -ac 1300060 -idNum 0 0 -add "Wrong /etc/chrony.conf" 2>/dev/null >/dev/null
				else
			        	echo "No diagtool installed on system - problem reporting / retreating alarm" | $LOGGER 2>/dev/null
				fi

				touch /tmp/1300060_alarm_off
				if [ -f /tmp/1300060_alarm_on ]
				then
					rm -f /tmp/1300060_alarm_on
				fi

				if [ "$DEBUG" == "yes" ]
				then
					echo "Retreat alarm for configuration file - /etc/chrony.conf" >>$LOG_FILE
				fi
			fi
		fi
	else
		# Configuration file does not exists
		if [ -e /opt/iskratel/bin/diagtool ]
		then
			if [ ! -f /tmp/1300060_alarm_on ]
			then
				if [ -e /opt/iskratel/bin/diagtool ]
				then
					#Report alarm for missing configuration file"
					/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300060 -idNum 0 0 -add "Missing /etc/chrony.conf" 2>/dev/null >/dev/null
				else
			        	echo "No diagtool installed on system - problem reporting / retreating alarm" | $LOGGER 2>/dev/null
				fi

				touch /tmp/1300060_alarm_on
				if [ -f /tmp/1300060_alarm_off ]
				then
					rm -f /tmp/1300060_alarm_off
				fi

				if [ "$DEBUG" == "yes" ]
				then
					echo "Report alarm for configuration file - /etc/chrony.conf" >>$LOG_FILE
				fi

                                #Clear previously reported 1300030 if exists
                                if [ -f /tmp/1300030_alarm_on ]
                                then
                                        /opt/iskratel/bin/diagtool -env -ret -norpc -ac 1300030 -idNum 0 0 -add "Cleared because 1300060 was reported" 2>/dev/null >/dev/null
                                        mv /tmp/1300030_alarm_on /tmp/1300030_alarm_off

                                        if [ "$DEBUG" == "yes" ]
                                        then
                                                echo "Retreate previously reported 1300030" >>$LOG_FILE
                                        fi
                                fi

			fi
		fi
	fi
	return 0
}



#Check if we have any servers listed. Do this via /etc/ntp.conf
function ntpd_servers_listed()
{
	if [ -f /etc/ntp.conf ]
	then
		# Configuration file exists
		server_num_all=$(cat /etc/ntp.conf | grep server | wc -l)
		server_num_hashed=$(cat /etc/ntp.conf | grep server | awk '{print $1;}' | grep "#" | wc -l)
		server_num=$(echo $(($server_num_all - $server_num_hashed)))

		if [ $server_num -eq 0 ]
		then
			# there are no servers in configuration file
			if [ ! -f /tmp/1300060_alarm_on ]
			then
				if [ -e /opt/iskratel/bin/diagtool ]
				then
					#Report alarm for missing configuration file"
					/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300060 -idNum 0 0 -add "Wrong /etc/ntp.conf" 2>/dev/null >/dev/null
				else
			        	echo "No diagtool installed on system - problem reporting / retreating alarm" | $LOGGER 2>/dev/null
				fi

				touch /tmp/1300060_alarm_on
				if [ -f /tmp/1300060_alarm_off ]
				then
					rm -f /tmp/1300060_alarm_off
				fi

				if [ "$DEBUG" == "yes" ]
				then
					echo "No servers in configuration file - /etc/ntp.conf" >>$LOG_FILE
				fi
			fi
		else
			# Configuration file is OK
			if [ ! -f /tmp/1300060_alarm_off ]
			then
				if [ -e /opt/iskratel/bin/diagtool ]
				then
					#Retreat alarm for missing configuration file"
					/opt/iskratel/bin/diagtool -env -ret -norpc -ac 1300060 -idNum 0 0 -add "Wrong /etc/ntp.conf" 2>/dev/null >/dev/null
				else
			        	echo "No diagtool installed on system - problem reporting / retreating alarm" | $LOGGER 2>/dev/null
				fi

				touch /tmp/1300060_alarm_off
				if [ -f /tmp/1300060_alarm_on ]
				then
					rm -f /tmp/1300060_alarm_on
				fi

				if [ "$DEBUG" == "yes" ]
				then
					echo "Retreat alarm for configuration file - /etc/ntp.conf" >>$LOG_FILE
				fi
			fi
		fi
	else
		# Configuration file does not exists
		if [ -e /opt/iskratel/bin/diagtool ]
		then
			if [ ! -f /tmp/1300060_alarm_on ]
			then
				if [ -e /opt/iskratel/bin/diagtool ]
				then
					#Report alarm for missing configuration file"
					/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300060 -idNum 0 0 -add "Missing /etc/ntp.conf" 2>/dev/null >/dev/null
				else
			        	echo "No diagtool installed on system - problem reporting / retreating alarm" | $LOGGER 2>/dev/null
				fi

				touch /tmp/1300060_alarm_on
				if [ -f /tmp/1300060_alarm_off ]
				then
					rm -f /tmp/1300060_alarm_off
				fi

				if [ "$DEBUG" == "yes" ]
				then
					echo "Report alarm for configuration file - /etc/ntp.conf" >>$LOG_FILE
				fi
			fi
		fi
	fi
	return 0
}


# Check if servers listed in configuration are OK and handle alarms
function chronyd_servers_ok()
{
	#Check if servers are reacable and OK
	#But first, server must be running.
	#If alarm for server is on, than dont check for OK servers
	#If servers are not reachable or OK than report alarm 1300030

	#First check if alarm for NTP server is on (1300040)
	if [ -f /tmp/1300040_alarm_on ]
	then
		#Just return with 0
		if [ "$DEBUG" == "yes" ]
		then
			echo "chronyd process not running - skip OK check" >>$LOG_FILE
		fi

		return 0
	fi

	#chronyd is running - check servers via chronyc status
	chronyc -n sources > /tmp/NTPstatus

	#How many lines in result (first two are irrelevant)
	line_num=$(cat /tmp/NTPstatus | wc -l)

	#Parse result
	server_ok=0
	i=0
	while read line
	do
		#If this is line with matching result
		if [[ $line =~ "ns" ]] || [[ $line =~ "us" ]] || [[ $line =~ "ms" ]]
		then
			server_all=$(echo $line)
			#Second character tells us status of NTP server
			#If second character is "*" than server is sinchronized
			server_status=$(echo $server_all | awk '{print $1;}' | awk '{print substr($0,2,length($0)-1)}')
			if [ "$server_status" == "*" ]
			then
				#server is synchronized - break
				server_ok=1
				break
			else
				server_ok=0
			fi

		fi
		i=$((i+1))
	done < /tmp/NTPstatus

	server_name=$(echo $server_all | awk '{print $2}')
	#Alarm handling
	if [ "$server_ok" == 0 ]
	then
		#Report alarm
		if [ ! -f /tmp/1300030_alarm_on ]
		then
			if [ -e /opt/iskratel/bin/diagtool ]
			then
				#Report alarm. If server name is empy, report with empty string
                                if [ "$server_status" == "?" ]
                                then
					if [ "$server_name" == "" ]
					then
                                        	/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300030 -idNum 0 0 -add ""  2>/dev/null >/dev/null
					else
                                        	/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300030 -idNum 0 0 -add "$server_name - lost connectivity"  2>/dev/null >/dev/null
					fi
                                elif [ "$server_status" == "x" ]
                                then
					if [ "$server_name" == "" ]
					then
                                        	/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300030 -idNum 0 0 -add ""  2>/dev/null >/dev/null
					else
                                        	/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300030 -idNum 0 0 -add "$server_name - source false ticker"  2>/dev/null >/dev/null
					fi
                                elif [ "$server_status" == "~" ]
                                then
					if [ "$server_name" == "" ]
					then
                                        	/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300030 -idNum 0 0 -add ""  2>/dev/null >/dev/null
					else
                                        	/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300030 -idNum 0 0 -add "$server_name - source with too much variability"  2>/dev/null >/dev/null
					fi
                                else
					if [ "$server_name" == "" ]
					then
                                        	/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300030 -idNum 0 0 -add ""  2>/dev/null >/dev/null
					else
                                        	/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300030 -idNum 0 0 -add "$server_name - not yet ready or not responding"  2>/dev/null >/dev/null
					fi
                                fi

				FILE_SIZE=$(stat -c %s "/tmp/NTPhistory")
				if [ $FILE_SIZE -gt 100000 ]
				then
					rm /tmp/NTPhistory
				fi
				echo "**********************ALARM ON********************" >> /tmp/NTPhistory 2>/dev/null
				date >> /tmp/NTPhistory
				chronyc -n sources >> /tmp/NTPhistory
				echo "" >> /tmp/NTPhistory 2>/dev/null
			else
		        	echo "No diagtool installed on system - problem reporting / retreating alarm" | $LOGGER 2>/dev/null
			fi

			touch /tmp/1300030_alarm_on
			if [ -f /tmp/1300030_alarm_off ]
			then 
				rm -f /tmp/1300030_alarm_off
			fi

			if [ "$DEBUG" == "yes" ]
			then
				echo "Report alarm 1300030 - ntpd" >>$LOG_FILE
			fi
		fi
	else
		#Retreat alarm
		if [ ! -f /tmp/1300030_alarm_off ]
		then
			if [ -e /opt/iskratel/bin/diagtool ]
			then
				/opt/iskratel/bin/diagtool -env -ret -norpc -ac 1300030 -idNum 0 0 -add "$server_name - server response OK" 2>/dev/null >/dev/null
				FILE_SIZE=$(stat -c %s "/tmp/NTPhistory")
				if [ $FILE_SIZE -gt 100000 ]
				then
					rm /tmp/NTPhistory
				fi
				echo "**********************ALARM OFF********************" >> /tmp/NTPhistory  2>/dev/null 
				date >> /tmp/NTPhistory
				chronyc -n sources >> /tmp/NTPhistory
				echo "" >> /tmp/NTPhistory 2>/dev/null
			else
		        	echo "No diagtool installed on system - problem reporting / retreating alarm" | $LOGGER 2>/dev/null
			fi

			touch /tmp/1300030_alarm_off
			if [ -f /tmp/1300030_alarm_on ]
			then 
				rm -f /tmp/1300030_alarm_on
			fi

			if [ "$DEBUG" == "yes" ]
			then
				echo "Retreat alarm 1300030 - ntpd" >>$LOG_FILE
			fi
		fi
	fi
	return 0
}


# Check if servers listed in configuration are OK and handle alarms
function ntpd_servers_ok()
{
	#Check if servers are reacable and OK
	#But first, server must be running.
	#If alarm for server is on, than dont check for OK servers
	#If servers are not reachable or OK than report alarm 1300030

	#First check if alarm for NTP server is on (1300040)
	if [ -f /tmp/1300040_alarm_on ]
	then
		#Just return with 0
		if [ "$DEBUG" == "yes" ]
		then
			echo "ntpd process not running - skip OK check" >>$LOG_FILE
		fi

		return 0
	fi

	#ntpd is running - check servers via ntpq -p
        ntpq -p > /tmp/NTPstatus 2>/tmp/NTPerr

	#Check if ntpq -p command has timed out
        line_num=$(cat /tmp/NTPerr | grep "timed out" | wc -l)

        if [ "$line_num" == "1" ]
        then
                #Command ntpq has timed out, so restart ntpq
		echo "Command ntpq has timed out, so restart ntpq" >> $LOG_FILE
		restart_ntpd
	fi


	#How many lines in result (first two are irrelevant)
	line_num=$(cat /tmp/NTPstatus | wc -l)

	#Parse result
	server_ok=0
	i=0
	while read line
	do
		#First two lines are not interresting
		if [ $i -lt 2 ]
		then
			#do nothing
			:
		else
			server_all=$(echo $line | awk '{print $1;}')
			#First character tells us status of NTP server
			#If first character is "*" than server is sinchronized
			server_status=$(echo $server_all | cut -c -1)
			if [ "$server_status" == "*" ] 
			then
				#server is synchronized - break
				server_ok=1
				break
			else
				server_ok=0
			fi

		fi
		i=$((i+1))
	done < /tmp/NTPstatus


	#alarm handling
	if [ "$server_ok" == 0 ]
	then
		if [ ! -f /tmp/1300030_alarm_on ]
		then
			if [ -e /opt/iskratel/bin/diagtool ]
			then
				if [ "$server_status" == "_" ] || [ "$server_status" == "-" ]
				then
					/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300030 -idNum 0 0 -add "$server_all - source discarded by cluster algorithm"  2>/dev/null >/dev/null
				elif [ "$server_status" == "x" ]
				then
					/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300030 -idNum 0 0 -add "$server_all - source false ticker"  2>/dev/null >/dev/null
				else
					/opt/iskratel/bin/diagtool -env -rep -norpc -ac 1300030 -idNum 0 0 -add "$server_all - not yet ready or not responding"  2>/dev/null >/dev/null
				fi

                                FILE_SIZE=$(stat -c %s "/tmp/NTPhistory")
                                if [ $FILE_SIZE -gt 100000 ]
                                then
                                        rm /tmp/NTPhistory
                                fi
                                echo "**********************ALARM ON********************" >> /tmp/NTPhistory 2>/dev/null
                                date >> /tmp/NTPhistory
                                ntpq -p >> /tmp/NTPhistory
                                echo "" >> /tmp/NTPhistory 2>/dev/null

			else
				echo "No diagtool installed on system - problem reporting / retreating alarm" | $LOGGER 2>/dev/null
			fi

			touch /tmp/1300030_alarm_on
			if [ -f /tmp/1300030_alarm_off ]
			then
				rm -f /tmp/1300030_alarm_off
			fi

			if [ "$DEBUG" == "yes" ]
			then
				echo "Report alarm 1300030 - ntpd" >>$LOG_FILE
			fi
		fi
	#it is possible that ntpd have some problems. Try to restart ntpd after 10min
	restart_ntpd_10min
	fi

	if [ "$server_ok" == 1 ]
	then
		#Retreat alarm
		if [ ! -f /tmp/1300030_alarm_off ]
		then
			if [ -e /opt/iskratel/bin/diagtool ]
			then
				/opt/iskratel/bin/diagtool -env -ret -norpc -ac 1300030 -idNum 0 0 -add "$server_all - server response OK" 2>/dev/null >/dev/null

                                FILE_SIZE=$(stat -c %s "/tmp/NTPhistory")
                                if [ $FILE_SIZE -gt 100000 ]
                                then
                                        rm /tmp/NTPhistory
                                fi
                                echo "**********************ALARM OFF********************" >> /tmp/NTPhistory 2>/dev/null
                                date >> /tmp/NTPhistory
                                ntpq -p >> /tmp/NTPhistory
                                echo "" >> /tmp/NTPhistory 2>/dev/null
			else
		        	echo "No diagtool installed on system - problem reporting / retreating alarm" | $LOGGER 2>/dev/null
			fi

			touch /tmp/1300030_alarm_off
			if [ -f /tmp/1300030_alarm_on ]
			then 
				rm -f /tmp/1300030_alarm_on
			fi

			if [ "$DEBUG" == "yes" ]
			then
				echo "Retreat alarm 1300030 - ntpd" >>$LOG_FILE
			fi
		fi
	fi
	return 0
}



#After 10min restart ntpd
function restart_ntpd_10min ()
{
        CYCLE=$(cat /tmp/NTPcycle)
        if [ $CYCLE -gt 9 ]
        then
		#Additionally chec if server is in INIT mode
		SERVER_MODE=$(ntpq -p|sed -n 3p|awk '{print $2}')
		if [ "$SERVER_MODE" == ".INIT." ]
		then
                	restart_ntpd
			echo "No response from NTP server for 10min - restarting NTP" | $LOGGER 2>/dev/null
                	echo "0" > /tmp/NTPcycle
		fi
        else
                CYCLE=$((CYCLE + 1))
                echo "$CYCLE" > /tmp/NTPcycle
        fi
}



# Check if configuration file has changed and restart NTP service
function ntpd_conf_check ()
{
	if [ -f /etc/ntp.conf ]
	then
		# Check if there is any differenc in configuration
		cp /etc/ntp.conf /tmp/ntp.conf
		conf_diff=$(diff /tmp/ntp.conf /tmp/ntp.conf.1 | wc -l)
		if [ "$conf_diff" != "0" ]
		then
			# There is change in configuration file - restart ntpd
			restart_ntpd
			echo "NTP configuration has changed - restarting ntpd" | $LOGGER 2>/dev/null 
       		fi
		# Now put new configuration into old one for further comparison

		cp /tmp/ntp.conf /tmp/ntp.conf.1
	fi
}



# Check if configuration file has changed and restart NTP service
function chronyd_conf_check ()
{
	# Check if there is any differenc in configuration
	cp /etc/chrony.conf /tmp/chrony.conf
	conf_diff=$(diff /tmp/chrony.conf /tmp/chrony.conf.1 | wc -l)
	if [ "$conf_diff" != "0" ]
	then
		# There is change in configuration file - restart chronyd
		if type systemctl >/dev/null 2>/dev/null
		then
			systemctl restart chronyd
		else
			/etc/init.d/chronyd restart
			sleep 5
		fi
		echo "NTP configuration has changed - restarting chronyd" | $LOGGER 2>/dev/null 

		if [ "$DEBUG" == "yes" ]
		then
			echo "NTP configuration has changed - restarting chronyd" >>$LOG_FILE
		fi
	fi
	# Now put new configuration into old one for further comparison
	cp /tmp/chrony.conf /tmp/chrony.conf.1
}



#Check if NTP is on or off. Check it via /opt/iskratel/sbin/ncfmCli get ntp-state
function check_ntp_on_off ()
{
	if [ -e /opt/iskratel/sbin/ncfmCli ]
	then
		NTP_ADM_STATE=$(/opt/iskratel/sbin/ncfmCli get ntp-state 2>/dev/null)
		if [ $NTP_ADM_STATE == "0" ]
		then
			#NTP is off, so do not test it
			#But only on AN products
			if [ -f /etc/default/sys_profile ]
			then
				PRODUCT=$(cat /etc/default/sys_profile | grep DIAG_MOD_NAME | awk '{print $2}')
				if [ "$PRODUCT" == "DIAG_MOD_NAME=AN" ]
				then
					if [ "$DEBUG" == "yes" ]
					then
						echo "Break with test - NTP is off" >>$LOG_FILE
					fi

					#Break with test, but before that clear all alarms
					reset_all_alarms_1
					exit
				fi
			else
				echo "No /etc/default/sys_profile installed on system - problem how to check if NTP is on" | $LOGGER 2>/dev/null
				if [ "$DEBUG" == "yes" ]
				then
					echo "No /etc/default/sys_profile installed on system - problem how to check if NTP is on" >>$LOG_FILE
				fi
			fi
		else
			#NTP is on
			#But only on AN products
                        if [ -f /etc/default/sys_profile ]
                        then
                                PRODUCT=$(cat /etc/default/sys_profile | grep DIAG_MOD_NAME | awk '{print $2}')
                                if [ "$PRODUCT" == "DIAG_MOD_NAME=AN" ]
                                then
					#Do nothing for 6 minutes (cycles)
					CYCLE=$(cat /tmp/NTPdelay)
					if [ $CYCLE == "1" ]
					then
						#Prepare NTP servers - next cycle we start with test
						DAEMON=$(cat /etc/sysconfig/ntp-info | grep DEAMON | awk -F'=' '{print $2}')
						PID=$(pidof $DAEMON)
						if [ "$PID" == "" ]
						then
							/etc/init.d/$DAEMON start
						fi
					fi
					if [ "$CYCLE" != "0" ]
					then
						CYCLE=$((CYCLE - 1))
						echo "$CYCLE" > /tmp/NTPdelay
						exit
					fi
				fi
			fi
		fi
	else
		if [ "$DEBUG" == "yes" ]
		then
			echo "No ncfmCli installed on system - problem how to check if NTP is on" >>$LOG_FILE
		fi
	fi
}



# Restart ntpd (eqvivalent it_ntpd.sh)
function restart_ntpd
{
	NTPD_PID1=$(pidof ntpd)
	if [ -e /etc/init.d/it_ntp ]            # ISKRATEL init script exists
	then
		/etc/init.d/it_ntp stop           # run ISKRATEL init script
		/etc/init.d/it_ntp start           # run ISKRATEL init script
	elif [ -e /etc/init.d/ntpd ]
	then
		/etc/init.d/ntpd stop     # run original init script
		/etc/init.d/ntpd start     # run original init script
	else
		echo "Problem restarting ntpd" | $LOGGER 2>/dev/null
		if [ "$DEBUG" == "yes" ]
		then
			echo "Problem restarting ntpd" >>$LOG_FILE
		fi
	fi
	NTPD_PID2=$(pidof ntpd)
	if [ "$NTPD_PID1" == "$NTPD_PID2" ]
	then
		echo "Problem restarting ntpd (pid = $NTPD_PID1 - kill it on hard way" >>$LOG_FILE
		kill -9 kill -9 $NTPD_PID2
		sleep 1
		if [ -e /etc/init.d/it_ntp ]            # ISKRATEL init script exists
		then
			/etc/init.d/it_ntp start           # run ISKRATEL init script
		elif [ -e /etc/init.d/ntpd ]
		then
			/etc/init.d/ntpd start     # run original init script
		else
			echo "Still problem restarting ntpd (ntpd won't start)" >>$LOG_FILE
		fi
	fi

	#New start - clear old cycle counter
	echo "0" > /tmp/NTPcycle
}



# reset all alarms - NTP not enabled
function reset_all_alarms_1()
{
	rm -f /tmp/1300030_alarm_on
	rm -f /tmp/1300030_alarm_off
	rm -f /tmp/1300040_alarm_on
	rm -f /tmp/1300040_alarm_off
	rm -f /tmp/1300060_alarm_on
	rm -f /tmp/1300060_alarm_off
	
	if [ -e /opt/iskratel/bin/diagtool ]
	then
		/opt/iskratel/bin/diagtool -env -ret -norpc -ac 1300030 -idNum 0 0 2>/dev/null >/dev/null
		/opt/iskratel/bin/diagtool -env -ret -norpc -ac 1300040 -idNum 0 0 2>/dev/null >/dev/null
		/opt/iskratel/bin/diagtool -env -ret -norpc -ac 1300060 -idNum 0 0 2>/dev/null >/dev/null
	else
        	echo "No diagtool installed on system - problem reporting / retreating alarm" | $LOGGER 2>/dev/null
	fi
}


# Function prints help
function print_help()
{
	echo "Options:"
	echo "help              Print_help"
	echo "reset_alarms      Reset all alarms"
	echo "debug_on          Debug on"
	echo "debug_all_on      Debug on (set -x)"
	echo "debug_off         Debug off"
}


# For test purpuse reset all alarms
function reset_all_alarms()
{
	rm -f /tmp/1300030_alarm_on
	rm -f /tmp/1300030_alarm_off
	rm -f /tmp/1300040_alarm_on
	rm -f /tmp/1300040_alarm_off
	rm -f /tmp/1300060_alarm_on
	rm -f /tmp/1300060_alarm_off
                                                
	if [ -e /opt/iskratel/bin/diagtool ]
	then
		/opt/iskratel/bin/diagtool -env -ret -norpc -ac 1300030 -idNum 0 0 2>/dev/null >/dev/null
		/opt/iskratel/bin/diagtool -env -ret -norpc -ac 1300040 -idNum 0 0 2>/dev/null >/dev/null
		/opt/iskratel/bin/diagtool -env -ret -norpc -ac 1300060 -idNum 0 0 2>/dev/null >/dev/null
		echo "Reset all alarms from menu" | $LOGGER 2>/dev/null
	else
        	echo "No diagtool installed on system - problem reporting / retreating alarm" | $LOGGER 2>/dev/null
	fi
}


# Set debug on                         
function debug_on()
{
	touch /tmp/NTPdebug
}



# Set detailed debug on (set -x)
function debug_all_on()
{
	touch /tmp/NTPdebug_all
}



# Set debug off                         
function debug_off()
{
	if [ -f /tmp/NTPdebug ]
	then
		rm -f /tmp/NTPdebug
	fi

	if [ -f /tmp/NTPdebug_all ]
	then
		rm -f /tmp/NTPdebug_all
	fi
}




######################
# M A I N    B O D Y #
######################

if [ "$1" == "runTest" ]; then
shift

	# Parse command line if any
	argc=$#
	if [ $# -gt 1 ]
	then
		print_help
		exit
	fi

	command=$1
	case $command in
		help)
			print_help
			exit
			;;
		-h)
			print_help
			exit
			;;
		reset_alarms)
			reset_all_alarms
			exit
			;;
		debug_on)
			debug_on
			exit
			;;
		debug_all_on)
			debug_all_on
			exit
			;;
		debug_off)
			debug_off
			exit
			;;
	esac


	#Are we working in DEBUG mode?
	if [ -f /tmp/NTPdebug_all ]
	then
		DEBUG="yes"
		PS4=' $LINENO: '
		echo "" >> $LOG_FILE
		echo "" >> $LOG_FILE
		echo "" >> $LOG_FILE
		echo "========================================================" >> $LOG_FILE
		echo "execute $@ at `date +"%F %T.%N"`" >> $LOG_FILE
		set -x
		exec >> $LOG_FILE 2>&1

	elif [ -f /tmp/NTPdebug ]
	then
		DEBUG="yes"
	else
		DEBUG="no"
	fi

	if [ "$DEBUG" == "yes" ]
	then
		echo "" >> $LOG_FILE
		echo "" >> $LOG_FILE
		echo "" >> $LOG_FILE
		echo "========================================================" >> $LOG_FILE
		echo "execute $@ at `date +"%F %T.%N"`" >> $LOG_FILE
	fi

	#Toch some files needed later
	touch /tmp/ntp.conf
	touch /tmp/ntp.conf.1
	touch /tmp/chrony.conf
	touch /tmp/chrony.conf.1
	touch /tmp/NTPhistory

	#First check for system type and which daemon is running on sistem
	#Is this system-d or system-V type?
	typeChrony=0
	typed=$(ls -l /sbin/init | grep systemd | wc -l)
	typeV=$(ls -l /sbin/init | grep sysvinit | wc -l)
	if [ "$typed" == 1 ] && [ "$typeV" == 0 ]
	then
		systemd=yes
		systemV=no
		systemType="systemd"
	fi
	if [ "$typed" == 0 ] && [ "$typeV" == 1 ]
	then
		systemd=no
		systemV=yes
		systemType="systemV"

                # Check if there is Chrony running in case of systemV
		typeChrony=$(ls -l /etc/init.d | grep chrony | wc -l)
	fi

	if [ "$DEBUG" == "yes" ]
	then
		echo "System type is systemd=$systemd systemV=$systemV" >> $LOG_FILE
	fi


	#Optionaly check this two not allowed combination
	if [ "$typed" == 1 ] && [ "$typeV" == 1 ]
	then
		#We should not be here - check again on different way
		typed=$(ps --no-headers -o comm 1 | grep systemd | grep -v grep | grep -v iskratel | grep -v neroot | wc -l)
		typeV=$(ps --no-headers -o comm 1 | grep init | grep -v grep | grep -v iskratel | grep -v neroot | wc -l)

		if [ "$typed" != 0 ] && [ "$typeV" == 0 ]
		then
			systemd=yes
			systemV=no
			systemType="systemd"
		fi
		if [ "$typed" == 0 ] && [ "$typeV" != 0 ]
		then
			systemd=no
			systemV=yes
			systemType="systemV"
		fi

		if [ "$DEBUG" == "yes" ]
		then
			echo "System type is systemd=$systemd systemV=$systemV" >> $LOG_FILE
		fi

		if [ "$typed" == 1 ] && [ "$typeV" == 1 ]
		then
			if [ "$DEBUG" == "yes" ]
			then
				echo "ERROR - both system types" > /tmp/NTPstatus
				echo "ERROR - both system types" >> $LOG_FILE
			fi

			echo "ERROR - both system types"  | $LOGGER 2>/dev/null
			exit 100
		fi
	fi
	if [ "$typed" == 0 ] && [ "$typeV" == 0 ]
	then
                #We should not be here - check again on different way
                typed=$(ps --no-headers -o comm 1 | grep systemd | grep -v grep | grep -v iskratel | grep -v neroot | wc -l)
                typeV=$(ps --no-headers -o comm 1 | grep init | grep -v grep | grep -v iskratel | grep -v neroot | wc -l)

                if [ "$typed" != 0 ] && [ "$typeV" == 0 ]
                then
                        systemd=yes
                        systemV=no
                        systemType="systemd"
                fi
                if [ "$typed" == 0 ] && [ "$typeV" != 0 ]
                then
                        systemd=no
                        systemV=yes
                        systemType="systemV"
                fi

                if [ "$DEBUG" == "yes" ]
                then
                        echo "System type is systemd=$systemd systemV=$systemV" >> $LOG_FILE
                fi

		if [ "$typed" == 0 ] && [ "$typeV" == 0 ]
		then
			if [ "$DEBUG" == "yes" ]
			then
				echo "ERROR - no system type" > /tmp/NTPstatus
				echo "ERROR - no system type" >> $LOG_FILE
			fi
			echo "ERROR - no system type"  | $LOGGER 2>/dev/null 
			exit 100
		fi
	fi


	# Depending of system type, call appropriate functions
	# systemV = ntpd, systemd = chronyd
	# CML, CMK is mixture of two systems - exception!
	if [ -e /sbin/system-board-info ]
	then
		BOARD=$(/sbin/system-board-info --product-name)
		if [ "$BOARD" == "CML" ] || [ "$BOARD" == "CMK" ]
		then
			systemd=yes
			systemV=no
			systemType="systemd"
		fi
	fi
        #
        # Check if there is Chrony running and set systemd to yes as 
        # systemd is assumed as a synonym for Chrony :)
        #
	if [ "$typeChrony" == 1 ]
        then
			systemd=yes
			systemV=no
			systemType="systemd"
        fi

	case $systemType in
		systemd)
		        #Check if NTP is on
			check_ntp_on_off

			#Check if chronyd is running
			chronyd_is_running

			#Check if configuration has changed
			chronyd_conf_check

			#Check if there are any servers listed
			chronyd_servers_listed

			#Check if servers are reachable and OK
			chronyd_servers_ok
			;;
	
		systemV)
			#Check if NTP is on
			check_ntp_on_off

			#Check if ntpd is running
			ntpd_is_running

			#Check if configuration has changed
			ntpd_conf_check

			#Check if there are any servers listed
			ntpd_servers_listed

			#Check if servers are reachable and OK
			ntpd_servers_ok
			;;

		*)
			if [ "$DEBUG" == "yes" ]
			then
				echo "We should not be here - both system types" >> $LOG_FILE
			fi
			;;

	esac

#  end of tests
else
	/opt/iskratel/bin/template.FW $1 "$0 runTest" NTPTest&
	#This is first run - set timer cycle to 0
	echo "0" > /tmp/NTPcycle
	echo "6" > /tmp/NTPdelay
fi
